c*********************** BBdeskj **************************************
c     see files README and COPYING for      Copyright (C) B Bunk 4/1992
c     conditions of use and redistribution                V 2.0  2/2000
c----------------------------------------------------------------------
c     BBplot driver for HP DeskJet Plus (300 dpi)
c           (e.g. Olivetti JP 150)
c                 icc = 0 : letter quality (default)
c                       1 : draft
c
      program bbdesk
c
      parameter(dens=300.,nhmax=dens*8.,nvmax=3300)
      parameter(max4=(nhmax+31)/32)
      common/pixmat/c1mat(0:4*max4-1,0:nvmax-1)
      common/masks/c1mask(0:3,-1:31)
      character*1 c1mat,c1mask
c
      dimension ib(0:3)
c
      character*80 metfil,pixfil
      character*1  esc
      character    start*43,head*8,finish*13
      character*4  ctop,cleft,cline,cbot
      character*17 fmt
c
      logical rotate
c
c.....metafile format
1000  format(2i4)
c
c     prepare the bit masks
c
      ib(0)=0
      ib(1)=0
      ib(2)=0
      ib(3)=0
      imask=-1
c
10    continue
      c1mask(0,imask)=char(ib(0))
      c1mask(1,imask)=char(ib(1))
      c1mask(2,imask)=char(ib(2))
      c1mask(3,imask)=char(ib(3))
c
      imask=imask + 1
      do 20 i=0,3
      if(ib(i) .lt. 255)then
            ib(i)=2*ib(i) + 1
            goto 10
      endif
20    continue
c
c     input parameters
c
      call inpar(metfil,pixfil
     &          ,mleft,mright,mtop,lprint,lbase
     &          ,irot,ifeed,lwidth,nfeed,lxaxis,icc)
c
c     printer parameters (densities in dpi, sizes in mm)
c
      hsmax=nhmax * 25.4/dens
      vsmax=nvmax * 25.4/dens
c
c     printing area
c
      hsize=hsmax - mleft - mright
      vsize=lprint
      nv=vsize * dens/25.4
      if(hsize.lt.20.) goto 901
      if(vsize.lt.20.) goto 902
      if(nv.gt.nvmax)  goto 903
c
c     open the metafile
c
      open(10,file=metfil,form='FORMATTED',status='OLD',err=910)
      rewind(10)
c
      read(10,1000,end=911) ivers,idummy
      if(ivers .ne. -10  .and.  ivers .ne. -20) goto 912
c
c     metafile coordinate ranges
c
      read(10,1000,end=913) ix0,iy0
      if(ix0 .lt. 0) goto 913
c
      read(10,1000,end=913) ix1,iy1
      if(ix1 .lt. 0) goto 913
c
      read(10,1000,end=913) ixmax,iymax
      if(ixmax .lt. 0) goto 913
      xmin=0.
      xmax=ixmax
      ymin=0.
      ymax=iymax
c
      deltax=xmax-xmin
      deltay=ymax-ymin
      if(deltax .le. 0.) goto 914
      if(deltay .le. 0.) goto 915
      ytox=deltay/deltax
c
      xlen=ix1-ix0
      ylen=iy1-iy0
      if(xlen .le. 0.) goto 916
      if(ylen .le. 0.) goto 917
c
c     determine the plotting area
c
      if(lbase.gt.0 .or. lxaxis.gt.0)then
            xsize=lbase
            if(lxaxis.gt.0) xsize=lxaxis * deltax/xlen
            ysize=xsize*ytox
c
            rotate=irot.eq.1
            if(irot.eq.2)
     &            rotate=xsize.gt.hsize .or. ysize.gt.vsize
c
            if(rotate)then
                  if(xsize.gt.vsize .or. ysize.gt.hsize)goto 920
            else
                  if(xsize.gt.hsize .or. ysize.gt.vsize)goto 920
            endif
      else
            if(irot.eq.0)then
                  rotate=.false.
                  xsize=min(hsize,vsize/ytox)
            else if(irot.eq.1)then
                  rotate=.true.
                  xsize=min(vsize,hsize/ytox)
            else
                  xsize=min(hsize,vsize/ytox)
                  xsrot=min(vsize,hsize/ytox)
                  rotate=xsrot .gt. xsize
                  xsize=max(xsrot,xsize)
            endif
            ysize=xsize*ytox
      endif
c
      if(rotate)then
            nvpix=xsize * dens/25.4
            nhpix=ysize * dens/25.4
            skleft=.5*(hsize-ysize) + mleft
            skbot =.5*(vsize-xsize)
            sktop =skbot + mtop
      else
            nvpix=ysize * dens/25.4
            nhpix=xsize * dens/25.4
            skleft=.5*(hsize-xsize) + mleft
            skbot =.5*(vsize-ysize)
            sktop =skbot + mtop
      endif
      nbytes=(nhpix+7)/8
      hoff=(8*nbytes - nhpix) * 25.4/dens
      hmax=nhpix-1+hoff
      vmax=nvpix-1
c
c     mappings  metafile -> true -> pixel coordinates
c
      if(rotate)then
            sc =(nhpix-1)/deltay
            shx=-sc*xmin
            shy= sc*ymax + hoff
      else
            sc =(nhpix-1)/deltax
            shx=sc*ymax
            shy=sc*xmax + hoff
      endif
c
c     rescaling of the line width; lwidth=10 => use metafile value
c
      scw=.05*lwidth * sc
c
c     clear the pixel matrix
c
      do 90 i=0,nbytes-1
      do 90 j=0,nvpix-1
      c1mat(i,j)=char(0)
90    continue
c
c     mark the corners of the plotting area
c
      c1mat(       0,      0)=char(7)
      c1mat(       0,nvpix-1)=char(7)
      c1mat(nbytes-1,      0)=char(224)
      c1mat(nbytes-1,nvpix-1)=char(224)
c
c     read the metafile and fill the pixel matrix
c
100   continue
      read(10,1000,end=101)ix,iy
c
      if(ix .lt. 0)then
            dwpix=max( scw*iy , 0. )
            xpmax=vmax - dwpix
            ypmin=hoff + dwpix
            ypmax=hmax - dwpix
            if(ix .eq. -2) read(10,*,end=101)
            read(10,1000,end=101)ix,iy
            if(rotate)then
                  x1=shx + sc*ix
                  y1=shy - sc*iy
            else
                  x1=shx - sc*iy
                  y1=shy - sc*ix
            endif
            x1=min( max( x1 , dwpix ) , xpmax )
            y1=min( max( y1 , ypmin ) , ypmax )
            goto 100
      endif
c            
      if(rotate)then
            x2=shx + sc*ix
            y2=shy - sc*iy
      else
            x2=shx - sc*iy
            y2=shy - sc*ix
      endif
      x2=min( max( x2 , dwpix ) , xpmax )
      y2=min( max( y2 , ypmin ) , ypmax )
c
      call fill(x1,y1,x2,y2,dwpix)
c
      x1=x2
      y1=y2
      goto 100
101   continue
      close(10)
c
c     printer control sequences
c
      esc=char(27)
c
c --- reset - 300 dpi - normal (non-compact) mode
c           - lq or draft 
c           - skip top margin - set left margin
c           - start graphics
      iqual=50
      if(icc.eq.1) iqual=49
c
      ntop=sktop * dens/25.4
      do 150 i=4,1,-1
      ndiv=ntop/10
      ctop(i:i)=char(48 + ntop - 10*ndiv)
      ntop=ndiv
150   continue
c
      nleft=skleft * dens/25.4
      do 160 i=4,1,-1
      ndiv=nleft/10
      cleft(i:i)=char(48 + nleft - 10*ndiv)
      nleft=ndiv
160   continue
c
      start= esc//'*rB'
     &     //esc//'*t300R'
     &     //esc//'*b0M'
     &     //esc//'*r'//char(iqual)//'Q'
     &     //esc//'*p+'//ctop//'Y'
     &     //esc//'*p'//cleft//'X'
     &     //esc//'*r1A'
c
c --- header for graphics line
      n=nbytes
      do 170 i=4,1,-1
      ndiv=n/10
      cline(i:i)=char(48 + n - 10*ndiv)
      n=ndiv
170   continue
      head=esc//'*b'//cline//'W'
c
                  if(ifeed.eq.1)then
c --- finish graphics - form feed
      nfin=9
      finish(1:nfin)=esc//'*rB'//esc//'&l0H'
                  else
c --- finish graphics - skip bottom margin
c           [ note: subtract final line feed from bottom skip ]
      skbot=max( skbot - 25.4/6. , 0. )
      nbot=skbot * dens/25.4
      do 180 i=4,1,-1
      ndiv=nbot/10
      cbot(i:i)=char(48 + nbot - 10*ndiv)
      nbot=ndiv
180   continue
      nfin=13
      finish(1:nfin)=esc//'*rB'//esc//'*p+'//cbot//'Y'
                  endif
c
c     write the pixel matrix
c
      write(fmt,'(a,i4,a,i4,a)')
     & '(a,',nvpix,'(',nbytes+1,'a),a)'
c
      open(20,file=pixfil,err=930)
      rewind(20)
c
      write(20,fmt,err=931) start
     &   ,( head
     &     ,(c1mat(ibyte,irow),ibyte=nbytes-1,0,-1)
     &     , irow=0,nvpix-1 )
     &   ,finish(1:nfin)
c
      close(20)
c
900   stop 'BBdeskj: all done'
c
901   stop 'BBdeskj error: printing area is too narrow'
902   stop 'BBdeskj error: printing area is too short'
903   stop 'BBdeskj error: printing area is too long'
910   stop 'BBdeskj error: metafile not found'
911   stop 'BBdeskj error: metafile is empty'
912   stop 'BBdeskj error: unknown metafile version'
913   stop 'BBdeskj error: incomplete header in metafile'
914   stop 'BBdeskj error: zero x range in metafile'
915   stop 'BBdeskj error: zero y range in metafile'
916   stop 'BBdeskj error: zero x axis specification in metafile'
917   stop 'BBdeskj error: zero y axis specification in metafile'
920   stop 'BBdeskj error: plot does not fit into printing area'
930   stop 'BBdeskj error: cannot open pixelfile'
931   close(20,status='DELETE')
      stop 'BBdeskj error: cannot write pixelfile'
c
      end
c------------------------------------------------------------
c     fill a rectangle (arguments are in pixel coordinates)
c
      subroutine fill(x1,y1,x2,y2,s)
c
      rx=x2-x1
      ry=y2-y1
c
      if(abs(ry) .lt. .1)then
c
c           horizontal box
c
            xm=.5*(x1+x2)
            ix1=min( x1+.5 , x2+.5 , xm )
            ix2=max( x1-.5 , x2-.5 , xm )
c
            sd=.5*int(2.*s - .5)
            iy1=y1 - sd
            iy2=y1 + sd
c
            do 100 ix=ix1,ix2
            call column(ix,iy1,iy2)
100         continue
c
      else if(abs(rx) .lt. .1)then
c
c           vertical box
c
            sd=.5*int(2.*s - .5)
            ix1=x1 - sd
            ix2=x1 + sd
c
            ym=.5*(y1+y2)
            iy1=min( y1+.5 , y2+.5 , ym )
            iy2=max( y1-.5 , y2-.5 , ym )
c
            do 200 ix=ix1,ix2
            call column(ix,iy1,iy2)
200         continue
c
      else
c
c           general box
c
            rsq=rx**2 + ry**2
            r=sqrt(rsq)
            sx=-ry * s/r
            sy= rx * s/r
c
            if(rx.gt.0.)then
            if(ry.gt.0.)then
                  xa=x1+sx
                  yab=y1+sy
                  yad=yab
                  ybc=yab - 2.*s*r/rx
                  ydc=yab + rsq/ry
                  stepup=ry/rx
            else
                  xa=x1-sx
                  yab=y1-sy
                  yad=yab
                  ybc=yab + rsq/ry
                  ydc=yab + 2.*s*r/rx
                  stepup=-rx/ry
            endif
            else
            if(ry.gt.0.)then
                  xa=x2+sx
                  yab=y2+sy
                  yad=yab
                  ybc=yab - rsq/ry
                  ydc=yab - 2.*s*r/rx
                  stepup=-rx/ry
            else
                  xa=x2-sx
                  yab=y2-sy
                  yad=yab
                  ybc=yab + 2.*s*r/rx
                  ydc=yab - rsq/ry
                  stepup=ry/rx
            endif
            endif
c
            stepdn=-1./stepup
c
            ix=nint(xa)
            frac=min( ix+.5 , .5*(x1+x2) ) - xa
            yab=yab + frac*stepdn
            yad=yad + frac*stepup
            ybc=ybc + frac*stepup
            ydc=ydc + frac*stepdn
c
300         continue
            ybot=max(yab,ybc)
            ytop=min(yad,ydc)
c
            if(ytop.lt.ybot) return
c
            ym=.5*(ybot+ytop)
            iybot=min( ybot+.5 , ym )
            iytop=max( ytop-.5 , ym )
c
            call column(ix,iybot,iytop)
c
            ix=ix + 1
            yab=yab + stepdn
            yad=yad + stepup
            ybc=ybc + stepup
            ydc=ydc + stepdn
            goto 300
c
      endif
      end
c-----------------------------------------------------------
c     fill a column of pixels
c
      subroutine column(ix,iy1,iy2)
c
      parameter(dens=300.,nhmax=dens*8.,nvmax=3300)
      parameter(max4=(nhmax+31)/32)
      common/pixmat/i4mat(0:max4-1,0:nvmax-1)
      common/masks/mask(-1:31)
      integer*4 i4mat,mask
c
      iword1=iy1/32
      iword2=iy2/32
      jy1=iy1 - iword1*32
      jy2=iy2 - iword1*32
c
      jtop=min( jy2 , 31 )
      i4mat(iword1,ix)=ior( i4mat(iword1,ix) 
     &                    , ieor( mask(jtop) , mask(jy1-1) ) )
c
      do 100 iword=iword1+1,iword2
      jy2=jy2 - 32
      jtop=min( jy2 , 31 )
      i4mat(iword,ix)=ior( i4mat(iword,ix) , mask(jtop) )
100   continue
      end
c-------------------------------------------------------------
c     command line intepreter
c
c     built-in default values:
c     mleft  = 20
c     mright = 20
c     mtop   =  0
c     lprint =250
c     lbase  =  0
c     irot   =  2
c     ifeed  =  1
c     lwidth = 10
c     nfeed  = 48
c     lxaxis =  0
c     icc    =  0
c
      subroutine inpar(metfil,pixfil
     &                ,mleft,mright,mtop,lprint,lbase
     &                ,irot,ifeed,lwidth,nfeed,lxaxis,icc)
c
      character*80  metfil,pixfil
      character*160 line
      dimension ival(11)
      data ival/20,20,0,250,0,2,1,10,48,0,0/
c
      metfil=' '
      pixfil=' '
c
c     scan standard input for parameters and options
c
      npar=0
100   continue
      read(*,'(a)',end=101)line
c
      call decode(line,ikey,newval,i1,i2,npar)
c
      if(ikey.eq.0)then
            goto 100
      else if(ikey.gt.0)then
            ival(ikey)=newval
            if(ikey.eq. 5) ival(10)=0
            if(ikey.eq.10) ival( 5)=0
      else if(ikey.eq.-1)then
            metfil=line(i1:i2)
      else if(ikey.eq.-2)then
            pixfil=line(i1:i2)
      else
c
c           command file encountered
c
            open(40,file=line(i1:i2),status='OLD',err=960)
c
200         continue
            read(40,'(a)',end=201)line
c
            call decode(line,ikey,newval,i1,i2,npar)
c
            if(ikey.eq.0)then
                  goto 200
            else if(ikey.gt.0)then
                  ival(ikey)=newval
                  if(ikey.eq. 5) ival(10)=0
                  if(ikey.eq.10) ival( 5)=0
            else if(ikey.eq.-1)then
                  metfil=line(i1:i2)
            else if(ikey.eq.-2)then
                  pixfil=line(i1:i2)
            else
                  goto 950
            endif
c
            goto 200
c
201         close(40)
c
      endif
c
      goto 100
c
101   continue
      mleft  =ival(1)
      mright =ival(2)
      mtop   =ival(3)
      lprint =ival(4)
      lbase  =ival(5)
      irot   =ival(6)
      ifeed  =ival(7)
      lwidth =ival(8)
      nfeed  =ival(9)
      lxaxis =ival(10)
      icc    =ival(11)
c
      if(metfil.eq.' ') goto 951
      if(pixfil.eq.' ') goto 952
      if(irot.gt.2)     goto 953
      if(ifeed.gt.1)    goto 954
      if(icc.gt.1)      goto 955
c
      return
c
950   stop 'BBdeskj error: command files cannot be nested'
951   stop 'BBdeskj error: no metafile specified'
952   stop 'BBdeskj error: no pixelfile specified'
953   stop 'BBdeskj error: illegal value in option ro'
954   stop 'BBdeskj error: illegal value in option ff'
955   stop 'BBdeskj error: illegal value in option cc'
960   print'(3a)',
     &    ' BBdeskj error: command file ',line(i1:i2),' not found'
      stop ' '
      end
c............................................................
c     decode keywords, numerical values, and filenames
c           ikey=0      => line(1:77) or value is blank
c           ikey=1..11  => non-negative integer value in ival
c           ikey=-1..-3 => non-blank filename in line(i1:i2)
c
      subroutine decode(line,ikey,ival,i1,i2,npar)
c
      character*160 line
      character*2   skey(-3:11),ckey(-3:11)
      data skey/'cf','pf','mf','  '
     &         ,'lm','rm','tm','pl','bl','ro','ff','lw','ls','xl','cc'/
      data ckey/'CF','PF','MF','  '
     &         ,'LM','RM','TM','PL','BL','RO','FF','LW','LS','XL','CC'/
c
c     skip leading blanks
c
      do 100 i1=1,77
      if(line(i1:i1).ne.' ')goto 101
100   continue
      ikey=0
      return
101   continue
c
      if(line(i1:i1).eq.'-' .or. line(i1:i1).eq.'/')then
c
c           keyword encountered
c
            i1=i1+1
c
            do 200 ikey=-3,11
            if(ikey.eq.0)goto 200
            if(    line(i1:i1+1).eq.skey(ikey)
     &        .or. line(i1:i1+1).eq.ckey(ikey) )goto 201
200         continue
            goto 960
201         continue
            i1=i1+2
c
      else
c
c           parameter encountered
c
            npar=npar + 1
            if(npar.gt.2)goto 961
            ikey=-npar
c
      endif
c
c     skip leading blanks, colons, and equality signs
c
      j1=i1
      do 300 i1=j1,160
      if( line(i1:i1).ne.' ' .and. line(i1:i1).ne.':'
     &                       .and. line(i1:i1).ne.'=' )goto 301
300   continue
      ikey=0
      return
301   continue
c
      if(ikey.gt.0)then
c
c           return a numerical value
c
            ival=0
            ndig=0
            do 400 i=i1,160
            if(line(i:i).eq.' ') return
            idigit=ichar( line(i:i) ) - 48
            if(idigit.lt.0 .or. idigit.gt.9 .or. ndig.ge.6) goto 962
            ndig=ndig + 1
            ival=10*ival + idigit
400         continue
            goto 962
c
      else
c
c           return a filename
c
            do 410 i2=i1,min(i1+80,159)
            if(line(i2+1:i2+1).eq.' ') return
410         continue
            goto 963
c
      endif
c
960   print'(2a)',' BBdeskj error: unknown option ',line(i1:i1+1)
      stop ' '
961   stop         'BBdeskj error: too many paramters'
962   print'(2a)',' BBdeskj error: illegal value in option ',skey(ikey)
      stop ' '
963   stop         'BBdeskj error: filename is too long'
      end
