c*********************** BBdot24 **************************************
c     see files README and COPYING for      Copyright (C) B Bunk 3/1991
c     conditions of use and redistribution                V 2.0  2/2000
c----------------------------------------------------------------------
c     driver for 24-pin dot matrix printers:
c         - EpsonLQ 800 and IBM ProprinterX24 
c                 (e.g. StarLC 24-10)           -> icc = 0 or 1
c         - EpsonLQ 1500 (e.g. NEC P6)          -> icc = 0
c         - EpsonLQ 550                         -> icc = 1 or 2
c         - EpsonLQ 400                         -> icc = 2
c
      program dot24
c
      parameter(npins=24,hdens=180.,nhmax=1440,maxlin=76)
      parameter(max4=(3*maxlin+3)/4)
      common/pixmat/c1mat(0:4*max4-1,0:nhmax-1)
      common/masks/c1mask(0:3,-1:31)
      character*1 c1mat,c1mask
c
      dimension ib(0:3)
c
      character*80 metfil,pixfil
      character*1  esc,lf,crt,ff,setls*2
      character    start1*6,start2*7,head*5,tail*2,fin1*3,fin2*3
      character*21 fmt
c
      logical rotate
c
c.....metafile format
1000  format(2i4)
c
c     prepare the bit masks
c
      ib(0)=0
      ib(1)=0
      ib(2)=0
      ib(3)=0
      imask=-1
c
10    continue
      c1mask(0,imask)=char(ib(0))
      c1mask(1,imask)=char(ib(1))
      c1mask(2,imask)=char(ib(2))
      c1mask(3,imask)=char(ib(3))
c
      imask=imask + 1
      do 20 i=0,3
      if(ib(i) .lt. 255)then
            ib(i)=2*ib(i) + 1
            goto 10
      endif
20    continue
c
c     input parameters
c
      call inpar(metfil,pixfil
     &          ,mleft,mright,mtop,lprint,lbase
     &          ,irot,ifeed,lwidth,nfeed,lxaxis,icc)
c
c     printer parameters (densities in dpi, sizes in mm)
c
      hsmax=nhmax * 25.4/hdens
      vfeed=(nfeed/360.) * 25.4
      vsmax=maxlin * vfeed
      vdens=(360./nfeed) * npins
c
c     printing area
c
      hsize=hsmax - mleft - mright
      lines=nint(lprint/vfeed)
      vsize=lines*vfeed
      if(hsize.lt.20.)    goto 901
      if(vsize.lt.20.)    goto 902
      if(lines.gt.maxlin) goto 903
c
c     open the metafile
c
      open(10,file=metfil,form='FORMATTED',status='OLD',err=910)
      rewind(10)
c
      read(10,1000,end=911) ivers,idummy
      if(ivers .ne. -10  .and.  ivers .ne. -20) goto 912
c
c     metafile coordinate ranges
c
      read(10,1000,end=913) ix0,iy0
      if(ix0 .lt. 0) goto 913
c
      read(10,1000,end=913) ix1,iy1
      if(ix1 .lt. 0) goto 913
c
      read(10,1000,end=913) ixmax,iymax
      if(ixmax .lt. 0) goto 913
      xmin=0.
      xmax=ixmax
      ymin=0.
      ymax=iymax
c
      deltax=xmax-xmin
      deltay=ymax-ymin
      if(deltax .le. 0.) goto 914
      if(deltay .le. 0.) goto 915
      ytox=deltay/deltax
c
      xlen=ix1-ix0
      ylen=iy1-iy0
      if(xlen .le. 0.) goto 916
      if(ylen .le. 0.) goto 917
c
c     determine the plotting area
c
      if(lbase.gt.0 .or. lxaxis.gt.0)then
            xsize=lbase
            if(lxaxis.gt.0) xsize=lxaxis * deltax/xlen
            ysize=xsize*ytox
c
            rotate=irot.eq.1
            if(irot.eq.2)
     &            rotate=xsize.gt.hsize .or. ysize.gt.vsize
c
            if(rotate)then
                  if(xsize.gt.vsize .or. ysize.gt.hsize)goto 920
            else
                  if(xsize.gt.hsize .or. ysize.gt.vsize)goto 920
            endif
      else
            if(irot.eq.0)then
                  rotate=.false.
                  xsize=min(hsize,vsize/ytox)
            else if(irot.eq.1)then
                  rotate=.true.
                  xsize=min(vsize,hsize/ytox)
            else
                  xsize=min(hsize,vsize/ytox)
                  xsrot=min(vsize,hsize/ytox)
                  rotate=xsrot .gt. xsize
                  xsize=max(xsrot,xsize)
            endif
            ysize=xsize*ytox
      endif
c
      if(rotate)then
            nvpix=xsize * vdens/25.4
            nhpix=ysize * hdens/25.4
            skleft=.5*(hsize-ysize) + mleft
            skbot =.5*(vsize-xsize)
            sktop =skbot + mtop
      else
            nvpix=ysize * vdens/25.4
            nhpix=xsize * hdens/25.4
            skleft=.5*(hsize-xsize) + mleft
            skbot =.5*(vsize-ysize)
            sktop =skbot + mtop
      endif
      lines=(nvpix+npins-1)/npins
      voff=npins*lines - nvpix
      vmax=npins*lines - 1
      hmax=nhpix-1
c
c     mappings  metafile -> true -> pixel coordinates
c
      if(rotate)then
            scx= (nhpix-1)/deltay
            scy=-(nvpix-1)/deltax
            shx=-scx*ymin
            shy=-scy*xmax + voff
      else
            scx=(nhpix-1)/deltax
            scy=(nvpix-1)/deltay
            shx=-scx*xmin
            shy=-scy*ymin + voff
      endif
c
c     rescaling of the line width; lwidth=10 => use metafile value
c
      scw=.05*lwidth * scx
c
c     clear the pixel matrix
c
      do 90 i=0,3*lines-1
      do 90 j=0,nhpix-1
      c1mat(i,j)=char(0)
90    continue
c
c     mark the corners of the plotting area
c
      c1mat(        0,      0)=char(7)
      c1mat(        0,nhpix-1)=char(7)
      c1mat(3*lines-1,      0)=char(224)
      c1mat(3*lines-1,nhpix-1)=char(224)
c
c     read the metafile and fill the pixel matrix
c
100   continue
      read(10,1000,end=101)ix,iy
c
      if(ix .lt. 0)then
            dwpix=max( scw*iy , 0. )
            xpmax=hmax - dwpix
            ypmin=voff + dwpix
            ypmax=vmax - dwpix
            if(ix .eq. -2) read(10,*,end=101)
            read(10,1000,end=101)ix,iy
            if(rotate)then
                  x1=scx*iy + shx
                  y1=scy*ix + shy
            else
                  x1=scx*ix + shx
                  y1=scy*iy + shy
            endif
            x1=min( max( x1 , dwpix ) , xpmax )
            y1=min( max( y1 , ypmin ) , ypmax )
            goto 100
      endif
c            
      if(rotate)then
            x2=scx*iy + shx
            y2=scy*ix + shy
      else
            x2=scx*ix + shx
            y2=scy*iy + shy
      endif
      x2=min( max( x2 , dwpix ) , xpmax )
      y2=min( max( y2 , ypmin ) , ypmax )
c
      call fill(x1,y1,x2,y2,dwpix)
c
      x1=x2
      y1=y2
      goto 100
101   continue
      close(10)
c
c     printer control sequences
c
      esc=char(27)
      lf=char(10)
      crt=char(13)
      ff=char(12)
      unit=360.
      setls=char(28)//'3'
      if(icc.eq.1) setls=esc//'+'
      if(icc.eq.2)then
            unit=180.
            setls=esc//'3'
            nfeed=nfeed/2
      endif
c --- reset - skip top margin - set left margin
c           - set graphics line feed - crt
c           [note: the initial crt is dummy]
      m1=1
150   l1=(sktop * unit/25.4)/m1
      if(l1.gt.255)then
            m1=m1+1
            goto 150
      endif
      nleft=skleft * 10./25.4
      start1=crt
     &     //esc//'@'
     &     //setls//char(l1)
      start2=esc//char(108)//char(nleft)
     &     //setls//char(nfeed)
     &     //crt
c --- set graphics mode
      n2=nhpix/256
      n1=nhpix - 256*n2
      head=esc//'*'//char(39)//char(n1)//char(n2)
c --- end of graphics line
      tail=crt//lf
                  if(ifeed.eq.1)then
c --- reset the printer - form feed
      nfin1=2
      nfin2=1
      fin1(1:nfin1)=esc//'@'
      fin2(1:nfin2)=ff
      m2=0
                  else
c --- skip bottom margin - reset the printer
c           [ note: subtract vertical offset and final line feed
c                   from bottom skip ]
      skbot=max( skbot - voff*25.4/vdens - 25.4/6. , 0. )
      m2=1
160   l2=(skbot * unit/25.4)/m2
      if(l2.gt.255)then
            m2=m2+1
            goto 160
      endif
      nfin1=3
      nfin2=3
      fin1(1:nfin1)=setls//char(l2)
      fin2(1:nfin2)=esc//'@'//crt
                  endif
c
c     write the pixel matrix
c
      write(fmt,'(a,i4,a,i4,a)')
     &      '(20a,',lines,'(',3*nhpix+2,'a),20a)'
c
      open(20,file=pixfil,err=930)
      rewind(20)
c
      write(20,fmt,err=931) start1,(lf,i=1,m1),start2
     &   ,( head
     &     ,(c1mat(l3+2,j),c1mat(l3+1,j),c1mat(l3,j),j=0,nhpix-1)
     &     ,tail
     &            ,l3=3*lines-3,0,-3 )
     &   ,fin1(1:nfin1),(lf,i=1,m2),fin2(1:nfin2)
c
      close(20)
c
900   stop 'BBdot24: all done'
c
901   stop 'BBdot24 error: printing area is too narrow'
902   stop 'BBdot24 error: printing area is too short'
903   stop 'BBdot24 error: printing area is too long'
910   stop 'BBdot24 error: metafile not found'
911   stop 'BBdot24 error: metafile is empty'
912   stop 'BBdot24 error: unknown metafile version'
913   stop 'BBdot24 error: incomplete header in metafile'
914   stop 'BBdot24 error: zero x range in metafile'
915   stop 'BBdot24 error: zero y range in metafile'
916   stop 'BBdot24 error: zero x axis specification in metafile'
917   stop 'BBdot24 error: zero y axis specification in metafile'
920   stop 'BBdot24 error: plot does not fit into printing area'
930   stop 'BBdot24 error: cannot open pixelfile'
931   close(20,status='DELETE')
      stop 'BBdot24 error: cannot write pixelfile'
c
      end
c------------------------------------------------------------
c     fill a rectangle (arguments are in pixel coordinates)
c
      subroutine fill(x1,y1,x2,y2,s)
c
      rx=x2-x1
      ry=y2-y1
c
      if(abs(ry) .lt. .1)then
c
c           horizontal box
c
            xm=.5*(x1+x2)
            ix1=min( x1+.5 , x2+.5 , xm )
            ix2=max( x1-.5 , x2-.5 , xm )
c
            sd=.5*int(2.*s - .5)
            iy1=y1 - sd
            iy2=y1 + sd
c
            do 100 ix=ix1,ix2
            call column(ix,iy1,iy2)
100         continue
c
      else if(abs(rx) .lt. .1)then
c
c           vertical box
c
            sd=.5*int(2.*s - .5)
            ix1=x1 - sd
            ix2=x1 + sd
c
            ym=.5*(y1+y2)
            iy1=min( y1+.5 , y2+.5 , ym )
            iy2=max( y1-.5 , y2-.5 , ym )
c
            do 200 ix=ix1,ix2
            call column(ix,iy1,iy2)
200         continue
c
      else
c
c           general box
c
            rsq=rx**2 + ry**2
            r=sqrt(rsq)
            sx=-ry * s/r
            sy= rx * s/r
c
            if(rx.gt.0.)then
            if(ry.gt.0.)then
                  xa=x1+sx
                  yab=y1+sy
                  yad=yab
                  ybc=yab - 2.*s*r/rx
                  ydc=yab + rsq/ry
                  stepup=ry/rx
            else
                  xa=x1-sx
                  yab=y1-sy
                  yad=yab
                  ybc=yab + rsq/ry
                  ydc=yab + 2.*s*r/rx
                  stepup=-rx/ry
            endif
            else
            if(ry.gt.0.)then
                  xa=x2+sx
                  yab=y2+sy
                  yad=yab
                  ybc=yab - rsq/ry
                  ydc=yab - 2.*s*r/rx
                  stepup=-rx/ry
            else
                  xa=x2-sx
                  yab=y2-sy
                  yad=yab
                  ybc=yab + 2.*s*r/rx
                  ydc=yab - rsq/ry
                  stepup=ry/rx
            endif
            endif
c
            stepdn=-1./stepup
c
            ix=nint(xa)
            frac=min( ix+.5 , .5*(x1+x2) ) - xa
            yab=yab + frac*stepdn
            yad=yad + frac*stepup
            ybc=ybc + frac*stepup
            ydc=ydc + frac*stepdn
c
300         continue
            ybot=max(yab,ybc)
            ytop=min(yad,ydc)
c
            if(ytop.lt.ybot) return
c
            ym=.5*(ybot+ytop)
            iybot=min( ybot+.5 , ym )
            iytop=max( ytop-.5 , ym )
c
            call column(ix,iybot,iytop)
c
            ix=ix + 1
            yab=yab + stepdn
            yad=yad + stepup
            ybc=ybc + stepup
            ydc=ydc + stepdn
            goto 300
c
      endif
      end
c-----------------------------------------------------------
c     fill a column of pixels
c
      subroutine column(ix,iy1,iy2)
c
      parameter(npins=24,hdens=180.,nhmax=1440,maxlin=76)
      parameter(max4=(3*maxlin+3)/4)
      common/pixmat/i4mat(0:max4-1,0:nhmax-1)
      common/masks/mask(-1:31)
      integer*4 i4mat,mask
c
      iword1=iy1/32
      iword2=iy2/32
      jy1=iy1 - iword1*32
      jy2=iy2 - iword1*32
c
      jtop=min( jy2 , 31 )
      i4mat(iword1,ix)=ior( i4mat(iword1,ix) 
     &                    , ieor( mask(jtop) , mask(jy1-1) ) )
c
      do 100 iword=iword1+1,iword2
      jy2=jy2 - 32
      jtop=min( jy2 , 31 )
      i4mat(iword,ix)=ior( i4mat(iword,ix) , mask(jtop) )
100   continue
      end
c-------------------------------------------------------------
c     command line intepreter
c
c     built-in default values:
c     mleft  = 20
c     mright = 20
c     mtop   =  0
c     lprint =250
c     lbase  =  0
c     irot   =  2
c     ifeed  =  1
c     lwidth = 10
c     nfeed  = 48
c     lxaxis =  0
c     icc    =  0
c
      subroutine inpar(metfil,pixfil
     &                ,mleft,mright,mtop,lprint,lbase
     &                ,irot,ifeed,lwidth,nfeed,lxaxis,icc)
c
      character*80  metfil,pixfil
      character*160 line
      dimension ival(11)
      data ival/20,20,0,250,0,2,1,10,48,0,0/
c
      metfil=' '
      pixfil=' '
c
c     scan standard input for parameters and options
c
      npar=0
100   continue
      read(*,'(a)',end=101)line
c
      call decode(line,ikey,newval,i1,i2,npar)
c
      if(ikey.eq.0)then
            goto 100
      else if(ikey.gt.0)then
            ival(ikey)=newval
            if(ikey.eq. 5) ival(10)=0
            if(ikey.eq.10) ival( 5)=0
      else if(ikey.eq.-1)then
            metfil=line(i1:i2)
      else if(ikey.eq.-2)then
            pixfil=line(i1:i2)
      else
c
c           command file encountered
c
            open(40,file=line(i1:i2),status='OLD',err=960)
c
200         continue
            read(40,'(a)',end=201)line
c
            call decode(line,ikey,newval,i1,i2,npar)
c
            if(ikey.eq.0)then
                  goto 200
            else if(ikey.gt.0)then
                  ival(ikey)=newval
                  if(ikey.eq. 5) ival(10)=0
                  if(ikey.eq.10) ival( 5)=0
            else if(ikey.eq.-1)then
                  metfil=line(i1:i2)
            else if(ikey.eq.-2)then
                  pixfil=line(i1:i2)
            else
                  goto 950
            endif
c
            goto 200
c
201         close(40)
c
      endif
c
      goto 100
c
101   continue
      mleft  =ival(1)
      mright =ival(2)
      mtop   =ival(3)
      lprint =ival(4)
      lbase  =ival(5)
      irot   =ival(6)
      ifeed  =ival(7)
      lwidth =ival(8)
      nfeed  =ival(9)
      lxaxis =ival(10)
      icc    =ival(11)
c
      if(metfil.eq.' ') goto 951
      if(pixfil.eq.' ') goto 952
      if(irot.gt.2)     goto 953
      if(ifeed.gt.1)    goto 954
      if(icc.gt.2)      goto 955
c
      return
c
950   stop 'BBdot24 error: command files cannot be nested'
951   stop 'BBdot24 error: no metafile specified'
952   stop 'BBdot24 error: no pixelfile specified'
953   stop 'BBdot24 error: illegal value in option ro'
954   stop 'BBdot24 error: illegal value in option ff'
955   stop 'BBdot24 error: illegal value in option cc'
960   print'(3a)',
     &    ' BBdot24 error: command file ',line(i1:i2),' not found'
      stop ' '
      end
c............................................................
c     decode keywords, numerical values, and filenames
c           ikey=0      => line(1:77) or value is blank
c           ikey=1..11  => non-negative integer value in ival
c           ikey=-1..-3 => non-blank filename in line(i1:i2)
c
      subroutine decode(line,ikey,ival,i1,i2,npar)
c
      character*160 line
      character*2   skey(-3:11),ckey(-3:11)
      data skey/'cf','pf','mf','  '
     &         ,'lm','rm','tm','pl','bl','ro','ff','lw','ls','xl','cc'/
      data ckey/'CF','PF','MF','  '
     &         ,'LM','RM','TM','PL','BL','RO','FF','LW','LS','XL','CC'/
c
c     skip leading blanks
c
      do 100 i1=1,77
      if(line(i1:i1).ne.' ')goto 101
100   continue
      ikey=0
      return
101   continue
c
      if(line(i1:i1).eq.'-' .or. line(i1:i1).eq.'/')then
c
c           keyword encountered
c
            i1=i1+1
c
            do 200 ikey=-3,11
            if(ikey.eq.0)goto 200
            if(    line(i1:i1+1).eq.skey(ikey)
     &        .or. line(i1:i1+1).eq.ckey(ikey) )goto 201
200         continue
            goto 960
201         continue
            i1=i1+2
c
      else
c
c           parameter encountered
c
            npar=npar + 1
            if(npar.gt.2)goto 961
            ikey=-npar
c
      endif
c
c     skip leading blanks, colons, and equality signs
c
      j1=i1
      do 300 i1=j1,160
      if( line(i1:i1).ne.' ' .and. line(i1:i1).ne.':'
     &                       .and. line(i1:i1).ne.'=' )goto 301
300   continue
      ikey=0
      return
301   continue
c
      if(ikey.gt.0)then
c
c           return a numerical value
c
            ival=0
            ndig=0
            do 400 i=i1,160
            if(line(i:i).eq.' ') return
            idigit=ichar( line(i:i) ) - 48
            if(idigit.lt.0 .or. idigit.gt.9 .or. ndig.ge.6) goto 962
            ndig=ndig + 1
            ival=10*ival + idigit
400         continue
            goto 962
c
      else
c
c           return a filename
c
            do 410 i2=i1,min(i1+80,159)
            if(line(i2+1:i2+1).eq.' ') return
410         continue
            goto 963
c
      endif
c
960   print'(2a)',' BBdot24 error: unknown option ',line(i1:i1+1)
      stop ' '
961   stop         'BBdot24 error: too many paramters'
962   print'(2a)',' BBdot24 error: illegal value in option ',skey(ikey)
      stop ' '
963   stop         'BBdot24 error: filename is too long'
      end
