c*********************** BBkyo ****************************************
c     see files README and COPYING for      Copyright (C) B Bunk 7/1991
c     conditions of use and redistribution                V 2.0  2/2000
c----------------------------------------------------------------------
c     driver for Kyocera laser printers
c         - input : BBplot metafile
c         - output: printer file containing PRESCRIBE commands
c
      program bbkyo
c
      parameter(hsmax=200.,vsmax=287.,maxfil=80)
c
      character*(maxfil) metfil,prtfil
      logical rotate
c
c.....metafile format
1000  format(2i4)
c
c     input parameters
c
      call inpar(metfil,prtfil
     &          ,mleft,mright,mtop,lprint,lbase
     &          ,irot,ifeed,lwidth,nfeed,lxaxis,icc)
c
c     printing area
c
      hsize=hsmax - mleft - mright - .2
      vsize=lprint
      if(hsize.lt.20.) goto 901
      if(vsize.lt.20.) goto 902
      if((mtop+vsize).gt.vsmax) goto 903
      vsize=vsize - .2
c
c     open the metafile
c
      open(10,file=metfil,form='FORMATTED',status='OLD',err=910)
      rewind(10)
c
      read(10,1000,end=911) ivers,idummy
      if(ivers .ne. -10  .and.  ivers .ne. -20) goto 912
c
c     metafile coordinate ranges
c
      read(10,1000,end=913) ix0,iy0
      if(ix0 .lt. 0) goto 913
c
      read(10,1000,end=913) ix1,iy1
      if(ix1 .lt. 0) goto 913
c
      read(10,1000,end=913) ixmax,iymax
      if(ixmax .lt. 0) goto 913
      xmin=0.
      xmax=ixmax
      ymin=0.
      ymax=iymax
c
      deltax=xmax-xmin
      deltay=ymax-ymin
      if(deltax .le. 0.) goto 914
      if(deltay .le. 0.) goto 915
      ytox=deltay/deltax
c
      xlen=ix1-ix0
      ylen=iy1-iy0
      if(xlen .le. 0.) goto 916
      if(ylen .le. 0.) goto 917
c
c     determine the plotting area
c
      if(lbase.gt.0 .or. lxaxis.gt.0)then
            xsize=lbase
            if(lxaxis.gt.0) xsize=lxaxis * deltax/xlen
            ysize=xsize*ytox
c
            rotate=irot.eq.1
            if(irot.eq.2)
     &            rotate=xsize.gt.hsize .or. ysize.gt.vsize
c
            if(rotate)then
                  if(xsize.gt.vsize .or. ysize.gt.hsize)goto 920
            else
                  if(xsize.gt.hsize .or. ysize.gt.vsize)goto 920
            endif
      else
            if(irot.eq.0)then
                  rotate=.false.
                  xsize=min(hsize,vsize/ytox)
            else if(irot.eq.1)then
                  rotate=.true.
                  xsize=min(vsize,hsize/ytox)
            else
                  xsize=min(hsize,vsize/ytox)
                  xsrot=min(vsize,hsize/ytox)
                  rotate=xsrot .gt. xsize
                  xsize=max(xsrot,xsize)
            endif
            ysize=xsize*ytox
      endif
c
      if(rotate)then
            skleft=.5*(hsize-ysize) + mleft
            sktop =.5*(vsize-xsize) + mtop
      else
            skleft=.5*(hsize-xsize) + mleft
            sktop =.5*(vsize-ysize) + mtop
      endif
c
c     mappings  metafile -> true -> printer coordinates
c
      sc=.1*xsize/deltax
      if(rotate)then
            shx=.1*skleft - sc*ymin
            shy=.1*sktop  - sc*xmin
      else
            shx=.1*skleft - sc*xmin
            shy=.1*sktop  + sc*ymax
      endif
c
c     rescaling of the line width; lwidth=10 => use metafile value
c
      scw=.1*lwidth * sc
c
c     open the printer file -- write initial commands
c
      open(20,file=prtfil,form='FORMATTED',err=930)
ccc      rewind(20)
      write(20,'(a)',err=931) ' !R! UNIT C; COPY 1; SPO P;'
c
c     translate polygon sequences
c
100   continue
      read(10,1000,end=101)ix,iy
c
      if(ix .lt. 0)then
            dw=scw*iy
            dw=min( max( dw , .01 ) , 1. )
            if(ix .eq. -2) read(10,*,end=101)
            read(10,1000,end=101)ix,iy
            if(rotate)then
                  x1=shx + sc*iy
                  y1=shy + sc*ix
            else
                  x1=shx + sc*ix
                  y1=shy - sc*iy
            endif
            write(20,1100,err=931)dw,x1,y1
1100        format(' SPD ',f5.3,';MZP ',f6.3,',',f6.3,';')
            goto 100
      endif
c            
      if(rotate)then
            x2=shx + sc*iy
            y2=shy + sc*ix
      else
            x2=shx + sc*ix
            y2=shy - sc*iy
      endif
      write(20,1200,err=931)x2,y2
1200  format(' DZP ',f6.3,',',f6.3,';')
c
      x1=x2
      y1=y2
      goto 100
101   continue
      close(10)
c
c     eject page
c
      if(ifeed.eq.1) write(20,'(a)',err=931) ' PAGE;'
c
c     reset commands -- close the printer file
c
      write(20,'(a)',err=931) ' UNIT D; SPD 3.; UNIT I; EXIT;'
      close(20)
c
900   stop 'BBkyo: all done'
c
901   stop 'BBkyo error: printing area is too narrow'
902   stop 'BBkyo error: printing area is too short'
903   stop 'BBkyo error: printing area is too long'
910   stop 'BBkyo error: metafile not found'
911   stop 'BBkyo error: metafile is empty'
912   stop 'BBkyo error: unknown metafile version'
913   stop 'BBkyo error: incomplete header in metafile'
914   stop 'BBkyo error: zero x range in metafile'
915   stop 'BBkyo error: zero y range in metafile'
916   stop 'BBkyo error: zero x axis specification in metafile'
917   stop 'BBkyo error: zero y axis specification in metafile'
920   stop 'BBkyo error: plot does not fit into printing area'
930   stop 'BBkyo error: cannot open printer file'
931   close(20,status='DELETE')
      stop 'BBkyo error: cannot write printer file'
c
      end
c-------------------------------------------------------------
c     command line intepreter
c
c     built-in default values:
c     mleft  = 20
c     mright = 20
c     mtop   = 20
c     lprint =250
c     lbase  =  0
c     irot   =  2
c     ifeed  =  1
c     lwidth = 10
c     nfeed  = 48
c     lxaxis =  0
c     icc    =  0
c
      subroutine inpar(metfil,pixfil
     &                ,mleft,mright,mtop,lprint,lbase
     &                ,irot,ifeed,lwidth,nfeed,lxaxis,icc)
c
      character*80  metfil,pixfil
      character*160 line
      dimension ival(11)
      data ival/20,20,20,250,0,2,1,10,48,0,0/
c
      metfil=' '
      pixfil=' '
c
c     scan standard input for parameters and options
c
      npar=0
100   continue
      read(*,'(a)',end=101)line
c
      call decode(line,ikey,newval,i1,i2,npar)
c
      if(ikey.eq.0)then
            goto 100
      else if(ikey.gt.0)then
            ival(ikey)=newval
            if(ikey.eq. 5) ival(10)=0
            if(ikey.eq.10) ival( 5)=0
      else if(ikey.eq.-1)then
            metfil=line(i1:i2)
      else if(ikey.eq.-2)then
            pixfil=line(i1:i2)
      else
c
c           command file encountered
c
            open(40,file=line(i1:i2),status='OLD',err=955)
c
200         continue
            read(40,'(a)',end=201)line
c
            call decode(line,ikey,newval,i1,i2,npar)
c
            if(ikey.eq.0)then
                  goto 200
            else if(ikey.gt.0)then
                  ival(ikey)=newval
                  if(ikey.eq. 5) ival(10)=0
                  if(ikey.eq.10) ival( 5)=0
            else if(ikey.eq.-1)then
                  metfil=line(i1:i2)
            else if(ikey.eq.-2)then
                  pixfil=line(i1:i2)
            else
                  goto 950
            endif
c
            goto 200
c
201         close(40)
c
      endif
c
      goto 100
c
101   continue
      mleft  =ival(1)
      mright =ival(2)
      mtop   =ival(3)
      lprint =ival(4)
      lbase  =ival(5)
      irot   =ival(6)
      ifeed  =ival(7)
      lwidth =ival(8)
      nfeed  =ival(9)
      lxaxis =ival(10)
      icc    =ival(11)
c
      if(metfil.eq.' ') goto 951
      if(pixfil.eq.' ') goto 952
      if(irot.gt.2)     goto 953
      if(ifeed.gt.1)    goto 954
c
      return
c
950   stop 'BBkyo error: command files cannot be nested'
951   stop 'BBkyo error: no metafile specified'
952   stop 'BBkyo error: no printer file specified'
953   stop 'BBkyo error: illegal value in option ro'
954   stop 'BBkyo error: illegal value in option ff'
955   print'(3a)',
     &    ' BBkyo error: command file ',line(i1:i2),' not found'
      stop ' '
      end
c............................................................
c     decode keywords, numerical values, and filenames
c           ikey=0      => line(1:77) or value is blank
c           ikey=1..11  => non-negative integer value in ival
c           ikey=-1..-3 => non-blank filename in line(i1:i2)
c
      subroutine decode(line,ikey,ival,i1,i2,npar)
c
      character*160 line
      character*2   skey(-3:11),ckey(-3:11)
      data skey/'cf','pf','mf','  '
     &         ,'lm','rm','tm','pl','bl','ro','ff','lw','ls','xl','cc'/
      data ckey/'CF','PF','MF','  '
     &         ,'LM','RM','TM','PL','BL','RO','FF','LW','LS','XL','CC'/
c
c     skip leading blanks
c
      do 100 i1=1,77
      if(line(i1:i1).ne.' ')goto 101
100   continue
      ikey=0
      return
101   continue
c
      if(line(i1:i1).eq.'-' .or. line(i1:i1).eq.'/')then
c
c           keyword encountered
c
            i1=i1+1
c
            do 200 ikey=-3,11
            if(ikey.eq.0)goto 200
            if(    line(i1:i1+1).eq.skey(ikey)
     &        .or. line(i1:i1+1).eq.ckey(ikey) )goto 201
200         continue
            goto 960
201         continue
            i1=i1+2
c
      else
c
c           parameter encountered
c
            npar=npar + 1
            if(npar.gt.2)goto 961
            ikey=-npar
c
      endif
c
c     skip leading blanks, colons, and equality signs
c
      j1=i1
      do 300 i1=j1,160
      if( line(i1:i1).ne.' ' .and. line(i1:i1).ne.':'
     &                       .and. line(i1:i1).ne.'=' )goto 301
300   continue
      ikey=0
      return
301   continue
c
      if(ikey.gt.0)then
c
c           return a numerical value
c
            ival=0
            ndig=0
            do 400 i=i1,160
            if(line(i:i).eq.' ') return
            idigit=ichar( line(i:i) ) - 48
            if(idigit.lt.0 .or. idigit.gt.9 .or. ndig.ge.6) goto 962
            ndig=ndig + 1
            ival=10*ival + idigit
400         continue
            goto 962
c
      else
c
c           return a filename
c
            do 410 i2=i1,min(i1+80,159)
            if(line(i2+1:i2+1).eq.' ') return
410         continue
            goto 963
c
      endif
c
960   print'(2a)',' BBkyo error: unknown option ',line(i1:i1+1)
      stop ' '
961   stop         'BBkyo error: too many paramters'
962   print'(2a)',' BBkyo error: illegal value in option ',skey(ikey)
      stop ' '
963   stop         'BBkyo error: filename is too long'
      end
