!***************************** adds ***********************************
!     see files README and COPYING for      Copyright (C) B Bunk 7/1991
!     conditions of use and redistribution                rev   12/2013
!----------------------------------------------------------------------
!     add points, functions, polygons, splines,
!           histograms, and text to the plot
! - - - - - - - - - - - - - - - - - - - - - - -
!     add a point

      subroutine addpt(x,dx,y,dy,isym)

      use bbplot_internal_data

      if(npts .ge. maxpts)then
         call bbrealloc_pts(npts+1, ierr)
         if (ierr /= 0) then
            write(6,*)'BBplot error in addpt: cannot allocate -- ignored'
            return
         endif
      endif

      npts=npts + 1
      xdat(npts)=x
      dxdat(npts)=abs(dx)
      ydat(npts)=y
      dydat(npts)=abs(dy)
      isyms(npts)=isym
      end
! - - - - - - - - - - - - - - - - - - - - - - -
!     add a function entry

      subroutine addfct(ifct,x1,x2,ltype)

      use bbplot_internal_data

      if(nfcts .ge. maxfct)then
         call bbrealloc_fct(nfcts+1, ierr)
         if (ierr /= 0) then
            write(6,*)'BBplot error in addfct: cannot allocate -- ignored'
            return
         endif
      endif

      nfcts=nfcts + 1
      ifcts(nfcts)=ifct
      xxf1(nfcts)=min(x1,x2)
      xxf2(nfcts)=max(x1,x2)
      ltypes(nfcts)=ltype
      cutfl(nfcts)=.false.
      end
! - - - - - - - - - - - - - - - - - - - - - - -
!     add a polygon

      subroutine addpol(n,xpol,ypol,ltype)

      use bbplot_internal_data

      dimension xpol(*),ypol(*)

      if(n .lt. 2) return

      if((npols+n) .gt. maxpol)then
         call bbrealloc_pol(npols+n, ierr)
         if (ierr /= 0) then
            write(6,*)'BBplot error in addpol: cannot allocate -- ignored'
            return
         endif
      endif

      do 100 i=1,n
      npols=npols + 1
      xpols(npols)=xpol(i)
      ypols(npols)=ypol(i)
      ltpols(npols)=-1
100   continue
      ltpols(npols)=max(ltype , 0)
      end
! - - - - - - - - - - - - - - - - - - - - - - -
!     add a straight line

      subroutine addlin(x1,y1,x2,y2,ltype)

      use bbplot_internal_data

      if((npols+2) .gt. maxpol)then
         call bbrealloc_pol(npols+2, ierr)
         if (ierr /= 0) then
            write(6,*)'BBplot error in addlin: cannot allocate -- ignored'
            return
         endif
      endif

      npols=npols + 1
      xpols(npols)=x1
      ypols(npols)=y1
      ltpols(npols)=-1
      npols=npols + 1
      xpols(npols)=x2
      ypols(npols)=y2
      ltpols(npols)=max(ltype , 0)
      end
! - - - - - - - - - - - - - - - - - - - - - - -
!     add a cubic spline

      subroutine addspl(n,xpol,ypol,ltype)

      use bbplot_internal_data

      dimension xpol(*),ypol(*)

      if(n .lt. 2) return

      if((npols+n) .gt. maxpol)then
         call bbrealloc_pol(npols+n, ierr)
         if (ierr /= 0) then
            write(6,*)'BBplot error in addspl: cannot allocate -- ignored'
            return
         endif
      endif

      delta=xpol(n)-xpol(1)
      do 200 i=1,n
      if(i.ge.2)then
            if(delta*(xpol(i)-xpol(i-1)) .le. 0.)then
                  write(6,*) &
         'BBplot error in addspl: invalid x ordering -- ignored'
                  return
            endif
      endif
      j=npols+i
      xpols(j)=xpol(i)
      ypols(j)=ypol(i)
      ltpols(j)=-3
200   continue
      npols=npols + n
      ltpols(npols)=max(ltype , 0)
      end
! - - - - - - - - - - - - - - - - - - - - - - -
!     add a histogram contour

      subroutine addhis(ncnt,nbins,x0,dx,ynorm,ltype)

      use bbplot_internal_data

      dimension ncnt(*)

      if(nbins .lt. 1) return

      if((npols+2*nbins+2) .gt. maxpol)then
         call bbrealloc_pol(npols+2*nbins+2, ierr)
         if (ierr /= 0) then
            write(6,*)'BBplot error in addhis: cannot allocate -- ignored'
            return
         endif
      endif

      xn=x0
      npols=npols + 1
      xpols(npols)=xn
      ypols(npols)=0.
      ltpols(npols)=-1
      do 300 ibin=1,nbins
      npols=npols + 1
      xpols(npols)=xn
      ypols(npols)=ynorm * ncnt(ibin)
      ltpols(npols)=-1
      xn=xn + dx
      npols=npols + 1
      xpols(npols)=xn
      ypols(npols)=ynorm * ncnt(ibin)
      ltpols(npols)=-1
300   continue
      npols=npols + 1
      xpols(npols)=xn
      ypols(npols)=0.
      ltpols(npols)=max(ltype , 0)
      end
! - - - - - - - - - - - - - - - - - - - - - - -
!     add a text entry

      subroutine addtxt(x,y,string)

      use bbplot_internal_data

      character(*) string

      if(nstrgs .ge. maxstr)then
         call bbrealloc_str(nstrgs+1, ierr)
         if (ierr /= 0) then
            write(6,*)'BBplot error in addtxt: cannot allocate -- ignored'
            return
         endif
      endif

      nstrgs=nstrgs + 1
      ntxts=ntxts + 1
      xtxts(ntxts)=x
      ytxts(ntxts)=y
      ictxts(ntxts)=0
      itxts(ntxts)=nstrgs
      strngs(nstrgs)=string
      end

!     new call format with color index

      subroutine add_text(x,y,string,icol)

      use bbplot_internal_data

      character(*) string

      if(nstrgs .ge. maxstr)then
         call bbrealloc_str(nstrgs+1, ierr)
         if (ierr /= 0) then
            write(6,*)'BBplot error in add_txt: cannot allocate -- ignored'
            return
         endif
      endif

      nstrgs=nstrgs + 1
      ntxts=ntxts + 1
      xtxts(ntxts)=x
      ytxts(ntxts)=y
      ictxts(ntxts)=icol
      itxts(ntxts)=nstrgs
      strngs(nstrgs)=string
      end
