!***************************** bballoc ********************************
!     see files README and COPYING for     Copyright (C) B Bunk 12/2013
!     conditions of use and redistribution               rev
!----------------------------------------------------------------------
!     manage dynamical (re)allocation of storage for
!        points, functions, polygons, and strings
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
subroutine bbrealloc_pts(nnew, ierr)

  ! reallocate storage for data points
  ! called with nnew > maxpts >= 0
  ! success: ierr == 0
  !          arrays are extended
  !          maxpts is updated
  ! error:   ierr /= 0

  use bbplot_internal_data

  interface
     subroutine bbrealloc_r(arr, maxold, maxnew, ierr)
       real, dimension(:), allocatable :: arr
     end subroutine bbrealloc_r
  end interface
  interface
     subroutine bbrealloc_i(arr, maxold, maxnew, ierr)
       integer, dimension(:), allocatable :: arr
     end subroutine bbrealloc_i
  end interface

  maxnew = max(32, 2*maxpts)
  do
     if (maxnew >= nnew) exit
     maxnew = 2*maxnew
  enddo

  ierr = 0
  call bbrealloc_r(xdat,  maxpts, maxnew, ierr)
  call bbrealloc_r(dxdat, maxpts, maxnew, ierr)
  call bbrealloc_r(ydat,  maxpts, maxnew, ierr)
  call bbrealloc_r(dydat, maxpts, maxnew, ierr)
  call bbrealloc_i(isyms, maxpts, maxnew, ierr)
  if (ierr == 0) maxpts = maxnew

end subroutine bbrealloc_pts
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
subroutine bbrealloc_fct(nnew, ierr)

  ! same for functions

  use bbplot_internal_data

  interface
     subroutine bbrealloc_r(arr, maxold, maxnew, ierr)
       real, dimension(:), allocatable :: arr
     end subroutine bbrealloc_r
  end interface
  interface
     subroutine bbrealloc_i(arr, maxold, maxnew, ierr)
       integer, dimension(:), allocatable :: arr
     end subroutine bbrealloc_i
  end interface
  interface
     subroutine bbrealloc_l(arr, maxold, maxnew, ierr)
       logical, dimension(:), allocatable :: arr
     end subroutine bbrealloc_l
  end interface

  maxnew = max(2, 2*maxfct)
  do
     if (maxnew >= nnew) exit
     maxnew = 2*maxnew
  enddo

  ierr = 0
  call bbrealloc_i(ifcts,  maxfct, maxnew, ierr)
  call bbrealloc_r(xxf1,   maxfct, maxnew, ierr)
  call bbrealloc_r(xxf2,   maxfct, maxnew, ierr)
  call bbrealloc_i(ltypes, maxfct, maxnew, ierr)
  call bbrealloc_l(cutfl,  maxfct, maxnew, ierr)
  if (ierr == 0) maxfct = maxnew

end subroutine bbrealloc_fct
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
subroutine bbrealloc_pol(nnew, ierr)

  ! same for polygons and splines

  use bbplot_internal_data

  interface
     subroutine bbrealloc_r(arr, maxold, maxnew, ierr)
       real, dimension(:), allocatable :: arr
     end subroutine bbrealloc_r
  end interface
  interface
     subroutine bbrealloc_i(arr, maxold, maxnew, ierr)
       integer, dimension(:), allocatable :: arr
     end subroutine bbrealloc_i
  end interface

  maxnew = max(128, 2*maxpol)
  do
     if (maxnew >= nnew) exit
     maxnew = 2*maxnew
  enddo

  ierr = 0
  call bbrealloc_r(xpols,  maxpol, maxnew, ierr)
  call bbrealloc_r(ypols,  maxpol, maxnew, ierr)
  call bbrealloc_i(ltpols, maxpol, maxnew, ierr)
  if (ierr == 0) maxpol = maxnew

end subroutine bbrealloc_pol
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
subroutine bbrealloc_str(nnew, ierr)

  ! same for shared string storage

  use bbplot_internal_data

  interface
     subroutine bbrealloc_r(arr, maxold, maxnew, ierr)
       real, dimension(:), allocatable :: arr
     end subroutine bbrealloc_r
  end interface
  interface
     subroutine bbrealloc_i(arr, maxold, maxnew, ierr)
       integer, dimension(:), allocatable :: arr
     end subroutine bbrealloc_i
  end interface
  interface
     subroutine bbrealloc_s(arr, maxold, maxnew, ierr)
       use bbplot_internal_data   ! for maxlen
       character(maxlen), dimension(:), allocatable :: arr
     end subroutine bbrealloc_s
  end interface

  maxnew = max(8, 2*maxstr)
  do
     if (maxnew >= nnew) exit
     maxnew = 2*maxnew
  enddo

  ierr = 0
  call bbrealloc_s(strngs, maxstr, maxnew, ierr)
  call bbrealloc_i(itits,  maxstr, maxnew, ierr)
  call bbrealloc_i(isubs,  maxstr, maxnew, ierr)
  call bbrealloc_i(legs,   maxstr, maxnew, ierr)
  call bbrealloc_i(ilegs,  maxstr, maxnew, ierr)
  call bbrealloc_i(itxts,  maxstr, maxnew, ierr)
  call bbrealloc_i(ictxts, maxstr, maxnew, ierr)
  call bbrealloc_r(xtxts,  maxstr, maxnew, ierr)
  call bbrealloc_r(ytxts,  maxstr, maxnew, ierr)
  if (ierr == 0) maxstr = maxnew

end subroutine bbrealloc_str
!----------------------------------------------------------------------
! (re)allocate real, integer, logical, and string arrays
! success: ierr is left unchanged
! error: ierr /= 0 is returned

subroutine bbrealloc_r(arr, maxold, maxnew, ierr)

  real, dimension(:), allocatable :: arr
  real :: temp(maxold)

  if (maxold > 0) then
     temp = arr
     deallocate(arr)
     allocate(arr(maxnew), stat=istat)
     arr(1:maxold) = temp
  else
     allocate(arr(maxnew), stat=istat)
  endif
  if (istat /= 0) ierr = istat
  
end subroutine bbrealloc_r
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
subroutine bbrealloc_i(arr, maxold, maxnew, ierr)

  integer, dimension(:), allocatable :: arr
  integer :: temp(maxold)

  if (maxold > 0) then
     temp = arr
     deallocate(arr)
     allocate(arr(maxnew), stat=istat)
     arr(1:maxold) = temp
  else
     allocate(arr(maxnew), stat=istat)
  endif
  if (istat /= 0) ierr = istat
  
end subroutine bbrealloc_i
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
subroutine bbrealloc_l(arr, maxold, maxnew, ierr)

  logical, dimension(:), allocatable :: arr
  logical :: temp(maxold)

  if (maxold > 0) then
     temp = arr
     deallocate(arr)
     allocate(arr(maxnew), stat=istat)
     arr(1:maxold) = temp
  else
     allocate(arr(maxnew), stat=istat)
  endif
  if (istat /= 0) ierr = istat
  
end subroutine bbrealloc_l
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
subroutine bbrealloc_s(arr, maxold, maxnew, ierr)

  use bbplot_internal_data   ! for maxlen
  
  character(maxlen), dimension(:), allocatable :: arr
  character(maxlen) :: temp(maxold)

  if (maxold > 0) then
     temp = arr
     deallocate(arr)
     allocate(arr(maxnew), stat=istat)
     arr(1:maxold) = temp
  else
     allocate(arr(maxnew), stat=istat)
  endif
  if (istat /= 0) ierr = istat

end subroutine bbrealloc_s
