!***************************** bbdefs *********************************
!     see files README and COPYING for      Copyright (C) B Bunk 2/2000
!     conditions of use and redistribution                rev   12/2013
!----------------------------------------------------------------------
!     define global variables
!     initialise them with default values
!           - same values as in iniplt
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      module bbplot_internal_data

      parameter(maxlen=200, maxcol=255)
      parameter(eps=1.e-5, huge=1.e30)
 
!     coordinate system

!.....y/x aspect ratio of the coordinate box [1]
      real :: ytox=1.
!.....y/x input and isoscale flag
      real :: ytoxin=1.
      integer :: iso=0
!.....x range and log flag
      real :: xrmin=-huge
      real :: xrmax= huge
      logical :: xlogfl=.false.
!.....y range and log flag
      real :: yrmin=-huge
      real :: yrmax= huge
      logical :: ylogfl=.false.
!.....line width for frame [2]
      real :: wframe=.003

      real, save :: xxmin,xxmax,yymin,yymax,xscale,yscale

!     x axis

!.....label flag
      logical :: xlbfl=.true.
!.....ticks [3..6]
      real :: septkx=.0095
      real :: siztkx=.03
      real :: srtkx=.65
      real :: wtkx=.002
!.....labels [7..10]
      integer :: minlbx=3
      real :: seplbx=.01
      real :: sizlbx=.03
      real :: wrlbx=.075
!.....legend text
      character(maxlen) :: xstrng=' '
!.....legend separation and letter size [11..13]
      real :: seplgx=.01
      real :: sizlgx=.03
      real :: wrlgx=.075

!     y axis

!.....label flag
      logical :: ylbfl=.true.
!.....ticks [14..17]
      real :: septky=.0095
      real :: siztky=.03
      real :: srtky=.65
      real :: wtky=.002
!.....labels [18..21]
      integer :: minlby=3
      real :: seplby=.01
      real :: sizlby=.03
      real :: wrlby=.075
!.....legend text
      character(maxlen) :: ystrng=' '
!.....legend separation and letter size [22..24]
      real :: seplgy=.01
      real :: sizlgy=.03
      real :: wrlgy=.075

!     points

!.....counters and arrays
      integer :: npts=0
      integer :: maxpts=0
      real,    dimension(:), allocatable, save :: xdat, dxdat, ydat, dydat
      integer, dimension(:), allocatable, save :: isyms      
!.....size (radius) and relative line width for symbols [25,26]
      real :: radpt=.01
      real :: wrpt=.060
!.....line width and rotation parameters for error bars [27..29]
      real :: wbar=.0020
      real :: robarx=.3
      real :: robary=.3

!     functions

!.....counters and arrays
      integer :: nfcts=0
      integer :: maxfct=0
      real,    dimension(:), allocatable, save :: xxf1, xxf2
      integer, dimension(:), allocatable, save :: ifcts, ltypes
!.....discretisation intervals [30]
      integer :: ninter=100
!.....cut flags
      logical, dimension(:), allocatable, save :: cutfl

!     polygons and splines

!.....counters and arrays
      integer :: npols=0
      integer :: maxpol=0
      real,    dimension(:), allocatable, save :: xpols, ypols
      integer, dimension(:), allocatable, save :: ltpols

!     lines for functions, polygons, and splines

!.....line width and style [31..35]
      real :: wlin1=.0020
      real :: wlin2=.0030
      real :: sizdsh=.02
      real :: sepdsh=.011
      real :: sepdot=.009

!     titles

!.....counter
      integer :: ntits=0
!.....line separation and letter size [36..38]
      real :: septit=.015
      real :: siztit=.035
      real :: wrtit=.075

!     sub-titles

!.....counter
      integer :: nsubs=0
!.....line separation and letter size [39..41]
      real :: sepsub=.012
      real :: sizsub=.030
      real :: wrsub=.075

!     legend for point symbols and line types

!.....counter
      integer :: nlegs=0
!.....position [42,43]
      real :: xlgpos=0.
      real :: ylgpos=0.
!.....symbol field size and separation [44,45]
      real :: sizfld=.07
      real :: sepfld=.01
!.....line seperation and letter size [46..48]
      real :: seplg=.01
      real :: sizlg=.025
      real :: wrlg=.075

!     text insertions

!.....counter
      integer :: ntxts=0
!.....letter size [49,50]
      real :: siztxt=.025
      real :: wrtxt=.075

!.....shared string storage -- counters and arrays
      integer :: nstrgs=0
      integer :: maxstr=0
      character(maxlen), dimension(:), allocatable, save :: strngs
      integer, dimension(:), allocatable, save :: itits,isubs &
                                          ,legs,ilegs,itxts,ictxts
      real,    dimension(:), allocatable, save :: xtxts, ytxts

!     color map (palette)

      parameter(irgb1=(255*256 +   0)*256 +   0)
      parameter(irgb2=(  0*256 + 255)*256 +   0)
      parameter(irgb3=(  0*256 +   0)*256 + 255)
      parameter(irgb4=(255*256 + 140)*256 +   0)
      parameter(irgb5=(255*256 +   0)*256 + 255)
      parameter(irgb6=(  0*256 + 255)*256 + 255)
      parameter(irgb7=(165*256 +  42)*256 +  42)

      integer, private :: i      ! for the implied do
      integer, dimension(0:maxcol) :: irgb = &
         (/0,irgb1,irgb2,irgb3,irgb4,irgb5,irgb6,irgb7,(0,i=8,maxcol)/)

!     detach mode of display window

      integer :: idet=0

      end module bbplot_internal_data
