!***************************** bboutp *********************************
!     see files README and COPYING for      Copyright (C) B Bunk 7/1991
!     conditions of use and redistribution                rev   12/2013
!----------------------------------------------------------------------
!     output operations

!     mode = 1 : screen display
!                 - calls display functions of bbdisp.xxx
!            2 : write a metafile
!            3 : write a postscript file
! - - - - - - - - - - - - - - - - - - - - - - - - - -
      module bbplot_internal_output_settings

      save mode,xfact,yfact,xsub,ysub,iunit,sc,shx,shy,wold,irgbo
  
      end module bbplot_internal_output_settings
! - - - - - - - - - - - - - - - - - - - - - - - - - -
      subroutine bbinit(mode1,fname,xmin,xmax,ymin,ymax,ytox)

      use bbplot_internal_output_settings

      character(*) fname
      logical opened

!.....metafile format
1000  format(2i4)

      mode=mode1

!     ignore leading and trailing blanks of fname
!     determine output unit

      if(mode .eq. 2  .or.  mode .eq. 3)then
            call bbtrim(fname,i1,i2)

            iunit=6
30          continue
            iunit=iunit + 1
            if(iunit.gt.99) &
                  stop 'BBplot error in bbinit: too many open files'
            inquire(iunit,opened=opened)
            if(opened) goto 30
      endif

!     transformation plot -> output coordinates
!     initialise output operations

      deltax=xmax-xmin
      deltay=ymax-ymin

      wold=0.
      irgbo=0

      if(mode .eq. 1)then

            call bbopen(xmin,xmax,ymin,ymax)

      else if(mode .eq. 2)then
            xfact=9999. / max(deltax,deltay)
            yfact=xfact
            xsub=xmin
            ysub=ymin

            open(iunit,file=fname(i1:i2),err=900)
            rewind(iunit)

            write(iunit,1000,err=910) -20, 0
            write(iunit,1000,err=910) int(-xfact*xsub), int(-yfact*ysub)
            write(iunit,1000,err=910) &
                        int(xfact*(1.-xsub)),  int(yfact*(ytox-ysub))
            write(iunit,1000,err=910) &
                        int(xfact*(xmax-xsub)),int(yfact*(ymax-ysub))
      else if(mode .eq. 3)then

!           margins and printing area (in mm)

            mleft=20
            mbot=27
            hsize=170.
            vsize=250.

            if(deltay/deltax .gt. vsize/hsize)then
                  sc=72./25.4*vsize/deltay
            else
                  sc=72./25.4*hsize/deltax
            endif
            shx=72./25.4*(mleft + .5*hsize) - sc*.5*(xmin+xmax)
            shy=72./25.4*(mbot  + .5*vsize) - sc*.5*(ymin+ymax)

            open(iunit,file=fname(i1:i2),err=950)
            rewind(iunit)

            write(iunit,'(a)',err=960) '%!PS-Adobe-1.0'
            write(iunit,'(2a)',err=960)'%%Title: ',fname(i1:i2)
            write(iunit,'(a)',err=960) '%%Creator: BBplot 2/2000'
            write(iunit,'(a,4i4)',err=960) '%%BoundingBox:' &
                  ,int(shx + sc*xmin) &
                  ,int(shy + sc*ymin) &
                  ,int(shx + sc*xmax) + 1 &
                  ,int(shy + sc*ymax) + 1
            write(iunit,'(a)',err=960) '% orientation: portrait'
            write(iunit,'(a)',err=960) '% warning: showpage included'
            write(iunit,'(a)',err=960) 'gsave'
            write(iunit,'(a)',err=960) 'newpath'
            write(iunit,'(a)',err=960) '/BBdict 5 dict def'
            write(iunit,'(a)',err=960) 'BBdict begin'
            write(iunit,'(a)',err=960) '/W {setlinewidth} def'
            write(iunit,'(a)',err=960) '/C {setrgbcolor} def'
            write(iunit,'(a)',err=960) '/M {moveto} def'
            write(iunit,'(a)',err=960) '/L {lineto} def'
            write(iunit,'(a)',err=960) '/S {stroke} def'
      else
            stop 'BBplot error in bbinit: unknown output mode'
      endif
      return

900   stop 'BBplot error in bboutp: cannot open metafile'
910   close(iunit,status='DELETE')
      stop 'BBplot error in bboutp: cannot write metafile'
950   stop 'BBplot error in bboutp: cannot open postscript file'
960   close(iunit,status='DELETE')
      stop 'BBplot error in bboutp: cannot write postscript file'
      end
! - - - - - - - - - - - - - - - - - - - - - - - - - -
!     draw a polygon: n points (x(i),y(i)) , line-width w

      subroutine bbpoly(n,x,y,w,icol)

      use bbplot_internal_data            ! for irgb()
      use bbplot_internal_output_settings

      dimension x(*),y(*)

!.....metafile format
1000  format(2i4)
1001  format(i8)

      if(n.lt.2) return

      if(mode .eq. 1)then

            if(w .ne. wold)then
                  call bblwid(w)
                  wold=w
            endif
            if(irgb(icol) .ne. irgbo)then
                  irg=irgb(icol)/256
                  ib=irgb(icol) - irg*256
                  ir=irg/256
                  ig=irg - ir*256
                  call bbcol(ir,ig,ib)
                  irgbo=irgb(icol)
            endif
            call bbmvto(x(1),y(1))
            do 100 i=2,n
            call bblnto(x(i),y(i))
100         continue

      else if(mode .eq. 2)then

            if(irgb(icol) .eq. irgbo)then
                  write(iunit,1000) -1 , int(xfact*w)
            else
                  write(iunit,1000) -2 , int(xfact*w)
                  write(iunit,1001) irgb(icol)
                  irgbo=irgb(icol)
            endif

            do 200 i=1,n
            write(iunit,1000,err=910) &
               int(xfact*(x(i)-xsub)) , int(yfact*(y(i)-ysub))
200         continue

      else

            if(w .eq. wold)then
                  write(iunit,'(a)',err=960) 'S'
            else
                  write(iunit,1050,err=960) sc*w
1050              format('S',f6.1,' W')
                  wold=w
            endif
            if(irgb(icol) .ne. irgbo)then
                  irg=irgb(icol)/256
                  ib=irgb(icol) - irg*256
                  ir=irg/256
                  ig=irg - ir*256
                  write(iunit,1060,err=960)ir/255.,ig/255.,ib/255.
1060              format(f5.3,2f6.3,' C')
                  irgbo=irgb(icol)
            endif
            write(iunit,1100,err=960)shx+sc*x(1),shy+sc*y(1)
1100        format(f5.1,f6.1,' M')
            do 300 i=2,n
            write(iunit,1200,err=960)shx+sc*x(i),shy+sc*y(i)
1200        format(f5.1,f6.1,' L')
300         continue

      endif
      return

910   close(iunit,status='DELETE')
      stop 'BBplot error in bboutp: cannot write metafile'
960   close(iunit,status='DELETE')
      stop 'BBplot error in bboutp: cannot write postscript file'
      end subroutine bbpoly
! - - - - - - - - - - - - - - - - - - - - - - - - - -
!     terminate output

      subroutine bbterm

      use bbplot_internal_data            ! for idet
      use bbplot_internal_output_settings

      if(mode .eq. 1)then
            call bbhold(idet)
      else if(mode .eq. 2)then
            close(iunit)
      else
            write(iunit,'(a)',err=960) 'S'
            write(iunit,'(a)',err=960) 'end'
            write(iunit,'(a)',err=960) 'showpage'
            write(iunit,'(a)',err=960) 'grestore'
            close(iunit)
      endif
      return

960   close(iunit,status='DELETE')
      stop 'BBplot error in bboutp: cannot write postscript file'
      end subroutine bbterm
