!***************************** BBplot *********************************
!     see files README and COPYING for      Copyright (C) B Bunk 7/1991
!     conditions of use and redistribution                rev   12/2013
!----------------------------------------------------------------------
!     generate the plot
!           - screen display
!           - metafile
!           - postscript file

! - - - - - - - - - - - - - - - - - - - - - -
!     screen display

subroutine view

      call bbplot(1, ' ')

end subroutine view

! - - - - - - - - - - - - - - - - - - - - - -
!     generate a metafile

subroutine meta(fname)

      character(*) fname

      if(fname .eq. ' ') return
      if(len(fname) .lt. 1) return

      call bbplot(2, fname)

end subroutine meta

! - - - - - - - - - - - - - - - - - - - - - -
!     generate a postscript file

subroutine psfile(fname)

      character(*) fname

      if(fname .eq. ' ') return
      if(len(fname) .lt. 1) return

      call bbplot(3, fname)

end subroutine psfile

! = = = = = = = = = = = = = = = = = = = = = =
subroutine bbplot(mode, fname)

      use bbplot_internal_data

      character(*) fname

      parameter(reduct=.6,sca12=1.+32./21.*reduct)
      parameter(sca1=(9.+16.*reduct)/21.,sca2=(12.+16.*reduct)/21.)

      logical getmin,getmax

      ! polygon work arrays for
      !  frame, functions, polygons and splines, legends
      real, dimension(max(npols,ninter+1,5)) :: xpol, ypol

!     x range

      if(xlogfl .and. xrmax.le.0.)then
            write(6,910)'x'
            return
      endif

      getmax=xrmax.ge.huge
      xxmax=xrmax
      if(getmax) xxmax=-huge

      getmin=xrmin.le.-huge .or. ( xlogfl .and. xrmin.le.0. )
      xxmin=xrmin
      if(getmin) xxmin=huge

      if(getmin.or.getmax)then

!.....preliminary y range
      yymax=yrmax
      yymin=yrmin
      if(ylogfl) yymin=max(yymin,1./huge)

!.....scan the points
      do 110 i=1,npts
      if(ydat(i).lt.yymin .or. ydat(i).gt.yymax) goto 110
      if(getmin) xxmin=min(xxmin,xdat(i)-dxdat(i))
      if(getmax) xxmax=max(xxmax,xdat(i)+dxdat(i))
110   continue

!.....scan polygons and splines
      do 115 i=1,npols
      if(ypols(i).lt.yymin .or. ypols(i).gt.yymax) goto 115
      if(getmin) xxmin=min(xxmin,xpols(i))
      if(getmax) xxmax=max(xxmax,xpols(i))
115   continue

!.....scan the functions
      do 120 i=1,nfcts
      if(cutfl(i))goto 120

      if(getmin) xxmin=min(xxmin,xxf1(i))
      if(getmax) xxmax=max(xxmax,xxf2(i))
120   continue

      xxdiff=xxmax - xxmin
      if(xxdiff .gt. 0.)then
            if(xlogfl)then
                  if(xxmax .le. 0.)then
                        write(6,910)'x'
                        return
                  else if(xxmin .lt. 0.)then
                        write(6,911)'x'
                        xxmin=min( -.1*xxmin , .1*xxmax )
                  else if(xxmin .eq. 0.)then
                        write(6,911)'x'
                        xxmin=.01*xxmax
                  endif
                  q=(xxmax/xxmin)**.05
                  if(getmin) xxmin=xxmin / q
                  if(getmax) xxmax=xxmax * q
            else
                  if(getmin) xxmin=xxmin - .05*xxdiff
                  if(getmax) xxmax=xxmax + .05*xxdiff
            endif
      else if(xxdiff .eq. 0.)then
            if(xlogfl)then
                  if(xxmax .le. 0.)then
                        write(6,910)'x'
                        return
                  else
                        write(6,912)'x'
                        if(getmin) xxmin=.1*xxmin
                        if(getmax) xxmax=10.*xxmax
                  endif
            else
                  write(6,912)'x'
                  if(getmin) xxmin=xxmin - 1.
                  if(getmax) xxmax=xxmax + 1.
            endif
      else
            write(6,913)'x'
            if(xlogfl)then
                  if(.not.getmin)then
                        xxmax=10.*xxmin
                  else if(.not.getmax)then
                        xxmin=.1*xxmax
                  else
                        xxmin=.1
                        xxmax=10.
                  endif
            else
                  if(.not.getmin)then
                        xxmax=xxmin + 1.
                  else if(.not.getmax)then
                        xxmin=xxmax - 1.
                  else
                        xxmin=0.
                        xxmax=1.
                  endif
            endif
      endif

      endif

!     y range

      if(ylogfl .and. yrmax.le.0.)then
            write(6,910)'y'
            return
      endif

      getmax=yrmax.ge. huge
      yymax=yrmax
      if(getmax) yymax=-huge

      getmin=yrmin.le.-huge .or. ( ylogfl .and. yrmin.le.0. )
      yymin=yrmin
      if(getmin) yymin= huge

      if(getmin.or.getmax)then

!.....scan the points
      do 150 i=1,npts
      if(xdat(i).lt.xxmin .or. xdat(i).gt.xxmax) goto 150
      if(getmin) yymin=min(yymin,ydat(i)-dydat(i))
      if(getmax) yymax=max(yymax,ydat(i)+dydat(i))
150   continue

!.....scan polygons and splines
      do 155 i=1,npols
      if(xpols(i).lt.xxmin .or. xpols(i).gt.xxmax) goto 155
      if(getmin) yymin=min(yymin,ypols(i))
      if(getmax) yymax=max(yymax,ypols(i))
155   continue

!.....scan the functions
      do 160 i=1,nfcts
      if(cutfl(i))goto 160

      ifct=ifcts(i)
      xx1=max(xxmin,xxf1(i))
      xx2=min(xxmax,xxf2(i))
      if(xx1 .ge. xx2)goto 160
      xx=xx1
      if(xlogfl)then
            xxfact=(xx2/xx1)**(1./ninter)
            do 165 j=0,ninter
            yy=pltfct(ifct,xx)
            if(getmin) yymin=min(yymin,yy)
            if(getmax) yymax=max(yymax,yy)
            xx=xx * xxfact
165         continue
      else
            xxstep=(xx2-xx1)/ninter
            do 170 j=0,ninter
            yy=pltfct(ifct,xx)
            if(getmin) yymin=min(yymin,yy)
            if(getmax) yymax=max(yymax,yy)
            xx=xx + xxstep
170         continue
      endif

160   continue

      yydiff=yymax - yymin
      if(yydiff .gt. 0.)then
            if(ylogfl)then
                  if(yymax .le. 0.)then
                        write(6,910)'y'
                        return
                  else if(yymin .lt. 0.)then
                        write(6,911)'y'
                        yymin=min( -.1*yymin , .1*yymax )
                  else if(yymin .eq. 0.)then
                        write(6,911)'y'
                        yymin=.01*yymax
                  endif
                  q=(yymax/yymin)**.05
                  if(getmin) yymin=yymin / q
                  if(getmax) yymax=yymax * q
            else
                  if(getmin) yymin=yymin - .05*yydiff
                  if(getmax) yymax=yymax + .05*yydiff
            endif
      else if(yydiff .eq. 0.)then
            if(ylogfl)then
                  if(yymax .le. 0.)then
                        write(6,910)'y'
                        return
                  else
                        write(6,912)'y'
                        if(getmin) yymin=.1*yymin
                        if(getmax) yymax=10.*yymax
                  endif
            else
                  write(6,912)'y'
                  if(getmin) yymin=yymin - 1.
                  if(getmax) yymax=yymax + 1.
            endif
      else
            write(6,913)'y'
            if(ylogfl)then
                  if(.not.getmin)then
                        yymax=10.*yymin
                  else if(.not.getmax)then
                        yymin=.1*yymax
                  else
                        yymin=.1
                        yymax=10.
                  endif
            else
                  if(.not.getmin)then
                        yymax=yymin + 1.
                  else if(.not.getmax)then
                        yymin=yymax - 1.
                  else
                        yymin=0.
                        yymax=1.
                  endif
            endif
      endif

      endif

!     aspect ratio, isoscale plot

      ytox=ytoxin

      if(iso.eq.1)then
            if(xlogfl.neqv.ylogfl)then
                  write(6,920)
                  return
            else if(xlogfl)then
                  ytox=log(yymax/yymin)/log(xxmax/xxmin)
            else
                  ytox=(yymax-yymin)/(xxmax-xxmin)
            endif
      else if(iso.eq.2)then
            if(xlogfl.neqv.ylogfl)then
                  write(6,920)
                  return
            else if(xlogfl)then
                  if(yymax/yymin .gt. (xxmax/xxmin)**ytox)then
                        pad=(yymax/yymin)**(1./ytox) / (xxmax/xxmin)
                        xxmax=xxmax * sqrt(pad)
                        xxmin=xxmin / sqrt(pad)
                  else
                        pad=(xxmax/xxmin)**ytox / (yymax/yymin)
                        yymax=yymax * sqrt(pad)
                        yymin=yymin / sqrt(pad)
                  endif
            else
                  if((yymax-yymin) .gt. ytox*(xxmax-xxmin))then
                        pad=(yymax-yymin)/ytox - (xxmax-xxmin)
                        xxmax=xxmax + .5*pad
                        xxmin=xxmin - .5*pad
                  else
                        pad=(xxmax-xxmin)*ytox - (yymax-yymin)
                        yymax=yymax + .5*pad
                        yymin=yymin - .5*pad
                  endif
            endif
      endif

!     HELENA+5   TABEA+7  BURKHARD55  MAMA+46  ZOE5 WALTER   OMA ROSEMARIE 65

!     scale factors data -> plot coordinates

      if(xlogfl)then
            xscale=1./log(xxmax/xxmin)
      else
            xscale=1./(xxmax - xxmin)
      endif
      if(ylogfl)then
            yscale=ytox / log(yymax/yymin)
      else
            yscale=ytox/(yymax - yymin)
      endif

!     determine the total plot size

      if(xlbfl)then
            call bblabx(xmin,xmax,ymin,ymax,.false.)
      else
            xmin=0.
            xmax=1.
            ymin=0.
            ymax=ytox
      endif
      if(ylbfl) call bblaby(xmin,xmax,ymin,ymax,.false.)

      if(xstrng.ne.' ') ymin=ymin - seplgx - sca12*sizlgx
      if(ystrng.ne.' ') xmin=xmin - seplgy - sca12*sizlgy

      if(ntits.gt.0) ymax=max( ymax , ytox + septit )
      ymax=ymax + ntits*( septit + sca12*siztit )
      ymin=ymin - nsubs*( sepsub + sca12*sizsub )

      tol=2.*wframe
      if(npts.gt.0) tol=2.*radpt
      xmin=min( xmin - wrlgy*sizlgy - 10.*eps ,      - tol )
      xmax=max( xmax + wrlgx*sizlgx + 10.*eps , 1.   + tol )
      ymin=min( ymin - wrsub*sizsub - 10.*eps ,      - tol )
      ymax=max( ymax + wrtit*siztit + 10.*eps , ytox + tol )

      slogo=.010
      wrlogo=.1
      xmax=max( xmax , 1. + wframe + (29./21.+wrlogo)*slogo )
! - - - - - - - - - - - - - - - - - - - - - - - - - -
!     initialise output operations

      call bbinit(mode,fname,xmin,xmax,ymin,ymax,ytox)
! - - - - - - - - - - - - - - - - - - - - - - - - - - -
!     draw the coordinate frame

      xpol(1)=0.
      xpol(2)=1.
      xpol(3)=1.
      xpol(4)=0.
      xpol(5)=0.
      ypol(1)=0.
      ypol(2)=0.
      ypol(3)=ytox
      ypol(4)=ytox
      ypol(5)=0.

      call bbpoly(5,xpol,ypol,wframe,0)

!     plot points with error bars

      do 2000 i=1,npts
      xx=xdat(i)
      yy=ydat(i)
      dxx=dxdat(i)
      dyy=dydat(i)
      if(xx.lt.xxmin .or. xx.gt.xxmax &
            .or. yy.lt.yymin .or. yy.gt.yymax) goto 2000

      call bbxx2x(xx,x)
      call bbyy2y(yy,y)

      call bbsymb(isyms(i),x,y,radpt,wrpt)

!     error bars in x

      if(dxx .eq. 0.)goto 2050
      x1=x + radpt
      x3=x - radpt
      call bbxx2x( min(xxmax,xx+dxx) , x2 )
      call bbxx2x( max(xxmin,xx-dxx) , x4 )
      if(x2.le.x1 .and. x3.le.x4)goto 2050
      y1=y
      y2=y
      y3=y
      y4=y

      if(robarx .eq. 0.)goto 2040
      if(xlogfl)then
            rexp=exp( radpt/xscale )
            xxup=(xx+dxx) * rexp
            xxdn=(xx-dxx) / rexp
      else
            xxup=xx + dxx + radpt/xscale
            xxdn=xx - dxx - radpt/xscale
      endif
      if(ylogfl)then
            rexp=exp( radpt/yscale )
            yyup=yy * rexp
            yydn=yy / rexp
            yytol=yy * sinh( 2.*wbar/yscale )
      else
            yyup=yy + radpt/yscale
            yydn=yy - radpt/yscale
            yytol=2.*wbar/yscale
      endif

      do 2010 j=1,npts
      if(xdat(j).lt.xxmin .or. xdat(j).gt.xxmax) goto 2010
      if( ydat(j).gt.yydn .and. ydat(j).lt.yyup )then
            if(xdat(j) .gt. xx)then
                  if( xdat(j).le.xxup &
                      .or. ( (xdat(j)-dxdat(j)).le.(xx+dxx) &
                             .and. abs(ydat(i)-yy).le.yytol ) )then
                        y2=y + robarx*(x2-x)
                        y2=min(y2,ytox)
                        y2=max(y2,0.)
                        y1=y + (y2-y)/(x2-x)*(x1-x)
                  endif
            else if(xdat(j) .lt. xx)then
                  if( xdat(j).ge.xxdn &
                      .or. ( (xdat(j)+dxdat(j)).ge.(xx-dxx) &
                             .and. abs(ydat(i)-yy).le.yytol ) )then
                        y4=y + robarx*(x4-x)
                        y4=min(y4,ytox)
                        y4=max(y4,0.)
                        y3=y + (y4-y)/(x4-x)*(x3-x)
                  endif
            endif
      endif
2010  continue

2040  continue

      if(x2.gt.x1) call bbline(x1,y1,x2,y2,wbar,isyms(i)/100)
      if(x3.gt.x4) call bbline(x3,y3,x4,y4,wbar,isyms(i)/100)

2050  continue

!     error bars in y

      if(dyy .eq. 0.)goto 2000
      y1=y + radpt
      y3=y - radpt
      call bbyy2y( min(yymax,yy+dyy) , y2 )
      call bbyy2y( max(yymin,yy-dyy) , y4 )
      if(y2.le.y1 .and. y3.le.y4)goto 2000
      x1=x
      x2=x
      x3=x
      x4=x

      if(robary .eq. 0.)goto 2090
      if(ylogfl)then
            rexp=exp( radpt/yscale )
            yyup=(yy+dyy) * rexp
            yydn=(yy-dyy) / rexp
      else
            yyup=yy + dyy + radpt/yscale
            yydn=yy - dyy - radpt/yscale
      endif
      if(xlogfl)then
            rexp=exp( radpt/xscale )
            xxup=xx * rexp
            xxdn=xx / rexp
            xxtol=xx * sinh( 2.*wbar/xscale )
      else
            xxup=xx + radpt/xscale
            xxdn=xx - radpt/xscale
            xxtol=2.*wbar/xscale
      endif

      do 2060 j=1,npts
      if(ydat(j).lt.yymin .or. ydat(j).gt.yymax) goto 2060
      if( xdat(j).gt.xxdn .and. xdat(j).lt.xxup )then
            if(ydat(j) .gt. yy)then
                  if( ydat(j).le.yyup &
                      .or. ( (ydat(j)-dydat(j)).le.(yy+dyy) &
                             .and. abs(xdat(i)-xx).le.xxtol ) )then
                        x2=x + robary*(y2-y)
                        x2=min(x2,1.)
                        x2=max(x2,0.)
                        x1=x + (x2-x)/(y2-y)*(y1-y)
                  endif
            else if(ydat(j) .lt. yy)then
                  if( ydat(j).ge.yydn &
                      .or. ( (ydat(j)+dydat(j)).ge.(yy-dyy) &
                             .and. abs(xdat(i)-xx).le.xxtol ) )then
                        x4=x + robary*(y4-y)
                        x4=min(x4,1.)
                        x4=max(x4,0.)
                        x3=x + (x4-x)/(y4-y)*(y3-y)
                  endif
            endif
      endif
2060  continue

2090  continue

      if(y2.gt.y1) call bbline(x1,y1,x2,y2,wbar,isyms(i)/100)
      if(y3.gt.y4) call bbline(x3,y3,x4,y4,wbar,isyms(i)/100)

2000  continue

!     plot functions

      do 2100 i=1,nfcts
      ifct=ifcts(i)
      ltype=ltypes(i)

      xx1=max(xxmin,xxf1(i))
      xx2=min(xxmax,xxf2(i))
      if(xx1 .ge. xx2)goto 2100
      xx=xx1
      npol=0
      if(xlogfl)then
            xxfact=(xx2/xx1)**(1./ninter)
      else
            xxstep =(xx2-xx1)/ninter
      endif
      do 2110 istep=0,ninter
      yy=pltfct(ifct,xx)
      if(yy.lt.yymin .or. yy.gt.yymax)then
            call bbdraw(npol,xpol,ypol,ltype)
            npol=0
      else
            npol=npol + 1
            call bbxx2x( xx , xpol(npol) )
            call bbyy2y( yy , ypol(npol) )
      endif
      if(xlogfl)then
            xx=xx * xxfact
      else
            xx=xx + xxstep
      endif
2110  continue
      call bbdraw(npol,xpol,ypol,ltype)

2100  continue

!     plot polygons and splines

      i1=1
      i2=0
      npol=0
      do 2150 i=1,npols
      if(i2.lt.i)then
2160        continue
            i2=i2 + 1
            ltype=ltpols(i2)
            if(ltype .lt. 0) goto 2160
      endif
      xx=xpols(i)
      yy=ypols(i)
      if(xx.ge.xxmin .and. xx.le.xxmax .and. &
         yy.ge.yymin .and. yy.le.yymax      )then
            npol=npol + 1
            call bbxx2x( xx , xpol(npol) )
            call bbyy2y( yy , ypol(npol) )
            if(i.lt.i2)goto 2150
      endif
      if(ltpols(i1) .eq. -1)then
            call bbdraw(npol,xpol,ypol,ltype)
      else
            call bbspl(npol,xpol,ypol,ltype,ninter,ytox)
      endif
      i1=i+1
      npol=0
2150  continue

!     label the axes

      if(xlbfl)then
            call bblabx(xleft,xright,ybot,ytop,.true.)
      else
            xleft=0.
            xright=1.
            ybot=0.
            ytop=ytox
      endif
      if(ylbfl) call bblaby(xleft,xright,ybot,ytop,.true.)

!     legends for the axes

      if(xstrng.ne.' ')then
            ybot=ybot - seplgx - sca2*sizlgx
            call bbtext(xstrng,.5,ybot,0.,1.,'c',xlen,sizlgx,wrlgx,0)
            ybot=ybot - sca1*sizlgx
      endif
      if(ystrng.ne.' ')then
            xleft=xleft - seplgy - sca1*sizlgy
            call bbtext(ystrng,xleft,.5*ytox,90.,ytox,'c' &
                                                ,xlen,sizlgy,wrlgy,0)
            xleft=xleft - sca2*sizlgy
      endif

!     legend for point symbols and line types

      if(nlegs .gt. 0)then

      if(xlgpos.le.0. .or. xlgpos.ge.(1.-sizfld-sepfld))then
            if(xlgpos.gt.0.) write(6,930)'x'
            xlgpos=1.5*abs(siztky) + .5*siztky
      endif
      if(ylgpos.le.0. .or. ylgpos.gt.ytox)then
            if(ylgpos.gt.0.) write(6,930)'y'
            ylgpos=ytox - 1.5*abs(siztkx) - .5*siztkx
      endif

      ylg=ylgpos
      xsym=xlgpos + .5*sizfld
      xtxt=xlgpos + sizfld + sepfld
      do 2200 i=1,nlegs
      ylg=ylg - sca12*sizlg
      if(ylg .lt. 0.)then
            write(6,931)
            goto 2290
      endif
      ytxt=ylg + sca1*sizlg
      j=legs(i)
      if(j .ge. 0)then
         if(mod(j,100) .ge. 50)then
            xpol(1)=xlgpos
            xpol(2)=xlgpos + sizfld
            ypol(1)=ytxt
            ypol(2)=ytxt
            call bbdraw(2,xpol,ypol,j-50)
         else
            call bbsymb(j,xsym,ytxt,radpt,wrpt)
         endif
      endif
      k=ilegs(i)
      call bbtext(strngs(k),xtxt,ytxt,0.,1.-xtxt,'l',xlen &
                                          ,sizlg,wrlg,abs(j)/100)
      ylg=ylg - seplg
2200  continue

      endif

2290  continue

!     titles and sub-titles

!cc      xc=.5*( xleft + xright )
!cc      xs=xright - xleft
      xc=.5
      xs=1.

      if(ntits.gt.0) ytop=max( ytop , ytox + septit )
      do 2300 i=ntits,1,-1
      ytop=ytop + septit + sca1*siztit
      k=itits(i)
      call bbtext(strngs(k),xc,ytop,0.,xs,'c',xlen,siztit,wrtit,0)
      ytop=ytop + sca2*siztit
2300  continue

      do 2310 i=1,nsubs
      ybot=ybot - sepsub - sca2*sizsub
      k=isubs(i)
      call bbtext(strngs(k),xc,ybot,0.,xs,'c',xlen,sizsub,wrsub,0)
      ybot=ybot - sca1*sizsub
2310  continue

!     insert text

      do 2400 i=1,ntxts
      xx=xtxts(i)
      yy=ytxts(i)
      if(xx.lt.xxmin .or. xx.gt.xxmax &
                  .or. yy.lt.yymin .or. yy.gt.yymax &
                  .or. sca12*siztxt.gt.ytox)then
            write(6,940)
            goto 2400
      endif
      call bbxx2x(xx,x)
      call bbyy2y(yy,y)
      y=max( y , sca1*siztxt )
      y=min( y , ytox-sca2*siztxt )
      k=itxts(i)
      call bbtext(strngs(k),x,y,0.,1.-x,'l',xlen,siztxt,wrtxt,ictxts(i))
2400  continue

      if(mode .ge. 2)then
            if(siztky .ge. 0.)then
               x=1. + wframe + 17./21.*slogo
               y=0.
               call bbtext('BBplot',x,y,-90.,.5,'r',xlen,slogo,wrlogo,0)
            else
               x=1. - wframe - 17./21.*slogo
               y=max(wframe,siztkx)
               call bbtext('BBplot',x,y,90.,.5,'l',xlen,slogo,wrlogo,0)
            endif
      endif
! - - - - - - - - - - - - - - - - - - - - - -
!     terminate output

      call bbterm
! - - - - - - - - - - - - - - - - - - - - - -
910   format(' BBplot error: invalid ',a,' range')
911   format(' BBplot warning: invalid ',a,' range -- adjusted')
912   format(' BBplot warning: zero ',a,' range -- expanded')
913   format(' BBplot warning: no ',a,' range -- use default')
920   format(' BBplot error: linlog plot cannot be isoscale')
930   format(' BBplot warning: legend out of ',a,' range -- adjusted')
931   format(' BBplot warning: legend out of space -- truncated')
940   format(' BBplot warning: text out of range -- ignored')

end subroutine bbplot
